package cn.fastfun.controller.api;

import cn.fastfun.controller.dto.asset.BaseInfoDTO;
import cn.fastfun.controller.dto.asset.ResponseDTO;
import cn.fastfun.controller.dto.asset.FigureDTO;
import cn.fastfun.controller.param.*;
import cn.fastfun.service.SysExcelFieldService;
import cn.fastfun.service.entity.AppAssetConf;
import cn.fastfun.service.impl.AppAssetConfServiceImpl;
import cn.fastfun.service.impl.AppAssetProfitServiceImpl;
import cn.fastfun.util.IrrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bridge.dto.*;
import com.bridge.exception.ApiRuntimeException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author Bridge AutoGen
 */
@Slf4j
@Api(tags = {"2.0 资产收益分析"})
@RestController
@RequestMapping("/api/v1/appassetprofit")
public class ApiAssetProfitController {

    @Autowired
    AppAssetProfitServiceImpl appAssetService;

    @Resource
    SysExcelFieldService sysExcelFieldService;

    @Autowired
    AppAssetConfServiceImpl appAssetConfService;


    @ApiOperation(value = "基础信息")
    @RequestMapping(value = "getBaseInfo", method = RequestMethod.POST)
    public ApiDTO getBaseInfo(@RequestBody @ApiParam(name = "设备信息对象", value = "传入json格式", required = true) AssetProfitParam param) throws Exception {

        if (!StringUtils.hasLength(param.getSn())) {
            throw new ApiRuntimeException("请输入sn");
        }
        BaseInfoDTO baseInfo = appAssetService.getBaseInfo(param);
        return ApiDTO.ok("成功", baseInfo);
    }

    @ApiOperation(value = "其他信息列表")
    @RequestMapping(value = "getProfitInfo", method = RequestMethod.POST)
    public ApiDTO getAssetProfitInfo(@RequestBody @ApiParam(name = "设备信息对象", value = "传入json格式", required = true) AssetProfitParam param) throws Exception {

        if (!StringUtils.hasLength(param.getSn())) {
            throw new ApiRuntimeException("请输入sn");
        }
        // 租赁信息
        Object response = null;
        if (param.getTableOrder() != null && param.getTableOrder().equals(1)) {
            param.setFigureOrNot(false);
            Object rentInfoDTO = appAssetService.getRentInfo(param);
            response = (ResponseDTO) rentInfoDTO;
        }
        // 运维信息
        else if (param.getTableOrder() != null && param.getTableOrder().equals(2)) {
            ResponseDTO responseDTO = new ResponseDTO();

            Page<AppAssetConf> appAssetConfPage = appAssetConfService.appAssetConfPageQuery(param);
            if (appAssetConfPage.getTotal() > 0) {
                appAssetConfPage.getRecords().forEach(p -> {
                    ResponseDTO.AllInfo allInfo = new ResponseDTO.AllInfo();
                    switch (p.getType()) {
                        case 1:
                            allInfo.setType("故障花费");
                            break;
                        case 2:
                            allInfo.setType("平台服务费");
                            break;
                        default:
                            break;
                    }
                    allInfo.setDescription("");
                    allInfo.setAmount(p.getAmount());
                    allInfo.setTime(new SimpleDateFormat("yyyy-MM-dd").format(p.getTime()));
                    responseDTO.getAllInfoList().add(allInfo);
                });
            }
            QueryWrapper<AppAssetConf> queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            if (param.getTimeStart() != null) {
                queryWrapper.gt("time", param.getTimeStart());
            }
            if (param.getTimeEnd() != null) {
                queryWrapper.lt("time", param.getTimeEnd());
            }
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.select("ifnull(sum(amount),0) as total");
            Map<String, Object> res = appAssetConfService.getMap(queryWrapper);
            responseDTO.setOmTotalCost(new BigDecimal(res.get("total").toString()));

            queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            if (param.getTimeStart() != null) {
                queryWrapper.gt("time", param.getTimeStart());
            }
            if (param.getTimeEnd() != null) {
                queryWrapper.lt("time", param.getTimeEnd());
            }
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 1);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            res = appAssetConfService.getMap(queryWrapper);
            responseDTO.setOmFaultCost(new BigDecimal(res.get("total").toString()));

            queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            if (param.getTimeStart() != null) {
                queryWrapper.gt("time", param.getTimeStart());
            }
            if (param.getTimeEnd() != null) {
                queryWrapper.lt("time", param.getTimeEnd());
            }
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 2);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            res = appAssetConfService.getMap(queryWrapper);
            responseDTO.setOmPlatformCost(new BigDecimal(res.get("total").toString()));

            response = (ResponseDTO) responseDTO;
        }

        // 赔偿信息
        else if (param.getTableOrder() != null && param.getTableOrder().equals(3)) {
            ResponseDTO responseDTO = new ResponseDTO();

            Page<AppAssetConf> appAssetConfPage = appAssetConfService.appAssetConfPageQuery(param);
            if (appAssetConfPage.getTotal() > 0) {
                appAssetConfPage.getRecords().forEach(p -> {
                    ResponseDTO.AllInfo allInfo = new ResponseDTO.AllInfo();
                    switch (p.getType()) {
                        case 1:
                            allInfo.setType("赔偿花费");
                            break;
                        default:
                            break;
                    }
                    allInfo.setDescription("");
                    allInfo.setAmount(p.getAmount());
                    allInfo.setTime(new SimpleDateFormat("yyyy-MM-dd").format(p.getTime()));
                    responseDTO.getAllInfoList().add(allInfo);
                });
            }
            QueryWrapper<AppAssetConf> queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            if (param.getTimeStart() != null) {
                queryWrapper.gt("time", param.getTimeStart());
            }
            if (param.getTimeEnd() != null) {
                queryWrapper.lt("time", param.getTimeEnd());
            }
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.select("ifnull(sum(amount),0) as total");
            Map<String, Object> res = appAssetConfService.getMap(queryWrapper);
            responseDTO.setComTotalCost(new BigDecimal(res.get("total").toString()));
            response = (ResponseDTO) responseDTO;
        }

        // 营销信息
        if (param.getTableOrder() != null && param.getTableOrder().equals(4)) {
            param.setFigureOrNot(false);
            Object rentInfoDTO = appAssetService.getSaleInfo(param);
            response = (ResponseDTO) rentInfoDTO;
        }

        // 运营信息
        else if (param.getTableOrder() != null && param.getTableOrder().equals(5)) {
            BaseInfoDTO baseInfo = (BaseInfoDTO) getBaseInfo(param).getData();
            ResponseDTO responseDTO = new ResponseDTO();

            Page<AppAssetConf> appAssetConfPage = appAssetConfService.appAssetConfPageQuery(param);
            if (appAssetConfPage.getTotal() > 0) {
                appAssetConfPage.getRecords().forEach(p -> {
                    ResponseDTO.AllInfo allInfo = new ResponseDTO.AllInfo();
                    switch (p.getType()) {
                        case 1:
                            allInfo.setType("仓储费用");
                            break;
                        case 2:
                            allInfo.setType("运输费用");
                            break;
                        case 3:
                            allInfo.setType("销售费用");
                            break;
                        case 4:
                            allInfo.setType("平台费用");
                            break;
                        default:
                            break;
                    }
                    allInfo.setDescription("");
                    allInfo.setAmount(p.getAmount());
                    allInfo.setTime(new SimpleDateFormat("yyyy-MM-dd").format(p.getTime()));
                    responseDTO.getAllInfoList().add(allInfo);
                });
            }
            if (baseInfo != null) {
                if (baseInfo.getStatus().equals(0) || baseInfo.getStatus().equals(14))
                    responseDTO.setOpStorageState("未出库");
                else
                    responseDTO.setOpStorageState("未出库");
            }

            param.setTableOrder(1);
            ResponseDTO assetProfitInfo = (ResponseDTO) getAssetProfitInfo(param).getData();
            if (assetProfitInfo != null) {
                responseDTO.setOpCurrentCash(assetProfitInfo.getCash() == null ? new BigDecimal(0) : assetProfitInfo.getCash());
            }


            param.setTableOrder(5);

            QueryWrapper<AppAssetConf> queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            if (param.getTimeStart() != null) {
                queryWrapper.gt("time", param.getTimeStart());
            }
            if (param.getTimeEnd() != null) {
                queryWrapper.lt("time", param.getTimeEnd());
            }
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.select("ifnull(sum(amount),0) as total");
            Map<String, Object> res = appAssetConfService.getMap(queryWrapper);
            responseDTO.setOpTotalCost(new BigDecimal(res.get("total").toString()));

            queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            if (param.getTimeStart() != null) {
                queryWrapper.gt("time", param.getTimeStart());
            }
            if (param.getTimeEnd() != null) {
                queryWrapper.lt("time", param.getTimeEnd());
            }
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 1);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            res = appAssetConfService.getMap(queryWrapper);
            responseDTO.setOpStorageCost(new BigDecimal(res.get("total").toString()));

            queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            if (param.getTimeStart() != null) {
                queryWrapper.gt("time", param.getTimeStart());
            }
            if (param.getTimeEnd() != null) {
                queryWrapper.lt("time", param.getTimeEnd());
            }
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 2);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            res = appAssetConfService.getMap(queryWrapper);
            responseDTO.setOpTransportCost(new BigDecimal(res.get("total").toString()));

            queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            if (param.getTimeStart() != null) {
                queryWrapper.gt("time", param.getTimeStart());
            }
            if (param.getTimeEnd() != null) {
                queryWrapper.lt("time", param.getTimeEnd());
            }
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 3);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            res = appAssetConfService.getMap(queryWrapper);
            responseDTO.setOpSaleCost(new BigDecimal(res.get("total").toString()));

            queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            if (param.getTimeStart() != null) {
                queryWrapper.gt("time", param.getTimeStart());
            }
            if (param.getTimeEnd() != null) {
                queryWrapper.lt("time", param.getTimeEnd());
            }
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 4);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            res = appAssetConfService.getMap(queryWrapper);
            responseDTO.setOpPlatformCost(new BigDecimal(res.get("total").toString()));
            response = (ResponseDTO) responseDTO;
        }

        // 金融信息
        else if (param.getTableOrder() != null && param.getTableOrder().equals(6)) {
            ResponseDTO responseDTO = new ResponseDTO();

            Page<AppAssetConf> appAssetConfPage = appAssetConfService.appAssetConfPageQuery(param);
            if (appAssetConfPage.getTotal() > 0) {
                appAssetConfPage.getRecords().forEach(p -> {
                    ResponseDTO.AllInfo allInfo = new ResponseDTO.AllInfo();
                    switch (p.getType()) {
                        case 1:
                            allInfo.setType("金融成本已付");
                            break;
                        case 2:
                            allInfo.setType("金融成本未付");
                            break;

                        default:
                            break;
                    }
                    allInfo.setDescription("");
                    allInfo.setAmount(p.getAmount());
                    allInfo.setTime(new SimpleDateFormat("yyyy-MM-dd").format(p.getTime()));
                    responseDTO.getAllInfoList().add(allInfo);
                });
            }
            QueryWrapper<AppAssetConf> queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            if (param.getTimeStart() != null) {
                queryWrapper.gt("time", param.getTimeStart());
            }
            if (param.getTimeEnd() != null) {
                queryWrapper.lt("time", param.getTimeEnd());
            }
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 1);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            Map<String, Object> res = appAssetConfService.getMap(queryWrapper);
            responseDTO.setFiPaidCost(new BigDecimal(res.get("total").toString()));

            queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            if (param.getTimeStart() != null) {
                queryWrapper.gt("time", param.getTimeStart());
            }
            if (param.getTimeEnd() != null) {
                queryWrapper.lt("time", param.getTimeEnd());
            }
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 2);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            res = appAssetConfService.getMap(queryWrapper);
            responseDTO.setFiResidueCost(new BigDecimal(res.get("total").toString()));
            response = (ResponseDTO) responseDTO;
        }

        // 处置信息
        else if (param.getTableOrder() != null && param.getTableOrder().equals(7)) {
            ResponseDTO responseDTO = new ResponseDTO();
            Page<AppAssetConf> appAssetConfPage = appAssetConfService.appAssetConfPageQuery(param);
            if (appAssetConfPage.getTotal() > 0) {
                appAssetConfPage.getRecords().forEach(p -> {
                    ResponseDTO.AllInfo allInfo = new ResponseDTO.AllInfo();
                    switch (p.getType()) {
                        case 1:
                            allInfo.setType("翻新金额-支");
                            break;
                        case 2:
                            allInfo.setType("处置支出");
                            break;
                        case 3:
                            allInfo.setType("处置收益");
                            break;
                        default:
                            break;
                    }
                    allInfo.setDescription("");
                    allInfo.setAmount(p.getAmount());
                    allInfo.setTime(new SimpleDateFormat("yyyy-MM-dd").format(p.getTime()));
                    responseDTO.getAllInfoList().add(allInfo);
                });
            }
            QueryWrapper<AppAssetConf> queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            if (param.getTimeStart() != null) {
                queryWrapper.gt("time", param.getTimeStart());
            }
            if (param.getTimeEnd() != null) {
                queryWrapper.lt("time", param.getTimeEnd());
            }
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.select("ifnull(sum(amount),0) as total");
            Map<String, Object> res = appAssetConfService.getMap(queryWrapper);
            responseDTO.setHandleProfit(new BigDecimal(res.get("total").toString()));

            response = (ResponseDTO) responseDTO;
        }

        // 其他信息
        else if (param.getTableOrder() != null && param.getTableOrder().equals(8)) {
            ResponseDTO responseDTO = new ResponseDTO();
            Page<AppAssetConf> appAssetConfPage = appAssetConfService.appAssetConfPageQuery(param);
            if (appAssetConfPage.getTotal() > 0) {
                appAssetConfPage.getRecords().forEach(p -> {
                    ResponseDTO.AllInfo allInfo = new ResponseDTO.AllInfo();
                    switch (p.getType()) {
                        case 1:
                            allInfo.setType("其他支出项目");
                            break;
                        case 2:
                            allInfo.setType("其他收入项目");
                            break;
                        default:
                            break;
                    }
                    allInfo.setDescription("");
                    allInfo.setAmount(p.getAmount());
                    allInfo.setTime(new SimpleDateFormat("yyyy-MM-dd").format(p.getTime()));
                    responseDTO.getAllInfoList().add(allInfo);
                });
            }
            QueryWrapper<AppAssetConf> queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            if (param.getTimeStart() != null) {
                queryWrapper.gt("time", param.getTimeStart());
            }
            if (param.getTimeEnd() != null) {
                queryWrapper.lt("time", param.getTimeEnd());
            }
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 1);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            Map<String, Object> res = appAssetConfService.getMap(queryWrapper);
            responseDTO.setOtherTotalCost(new BigDecimal(res.get("total").toString()));

            queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            if (param.getTimeStart() != null) {
                queryWrapper.gt("time", param.getTimeStart());
            }
            if (param.getTimeEnd() != null) {
                queryWrapper.lt("time", param.getTimeEnd());
            }
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 2);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            res = appAssetConfService.getMap(queryWrapper);
            responseDTO.setOtherTotalIncome(new BigDecimal(res.get("total").toString()));

            response = (ResponseDTO) responseDTO;
        }

        // 收入汇总信息
        else if (param.getTableOrder() != null && param.getTableOrder().equals(9)) {

            ResponseDTO responseDTO = new ResponseDTO();
            param.setTableOrder(1);
            param.setFigureOrNot(false);
            ResponseDTO rentInfo = (ResponseDTO) appAssetService.getRentInfo(param);
            if (rentInfo != null) {
                responseDTO.setNetRental(rentInfo.getNetRental());
                responseDTO.setCashIn(rentInfo.getCashIn());
                responseDTO.setSecurityCash(rentInfo.getSecurityCash());
            }

            param.setTableOrder(7);
            ResponseDTO assetHandleInfo = (ResponseDTO) getAssetProfitInfo(param).getData();
            if (assetHandleInfo != null) {
                responseDTO.setHandleProfit(assetHandleInfo.getHandleProfit());
            }

            param.setTableOrder(2);
            ResponseDTO assetOmInfo = (ResponseDTO) getAssetProfitInfo(param).getData();
            if (assetOmInfo != null) {
                responseDTO.setOmTotalCost(assetOmInfo.getOmTotalCost());
            }

            param.setTableOrder(3);
            ResponseDTO assetComInfo = (ResponseDTO) getAssetProfitInfo(param).getData();
            if (assetComInfo != null) {
                responseDTO.setComTotalCost(assetComInfo.getComTotalCost());
            }

            param.setTableOrder(4);
            ResponseDTO assetSaleInfo = (ResponseDTO) getAssetProfitInfo(param).getData();
            if (assetSaleInfo != null) {
                responseDTO.setSaleTotalCost(assetSaleInfo.getSaleTotalCost());
            }

            param.setTableOrder(5);
            ResponseDTO assetOpInfo = (ResponseDTO) getAssetProfitInfo(param).getData();
            if (assetOpInfo != null) {
                responseDTO.setOpTotalCost(assetOpInfo.getOpTotalCost());
            }

            param.setTableOrder(6);
            ResponseDTO assetFiInfo = (ResponseDTO) getAssetProfitInfo(param).getData();
            if (assetFiInfo != null) {
                responseDTO.setFiPaidCost(assetFiInfo.getFiPaidCost());
            }

            responseDTO.setNetIncome(responseDTO.getNetRental().add(responseDTO.getCashIn()).add(responseDTO.getSecurityCash()).add(responseDTO.getHandleProfit()).add(
                    responseDTO.getOmTotalCost()).add(responseDTO.getComTotalCost()).add(responseDTO.getSaleTotalCost()).add(responseDTO.getOpTotalCost()).add(
                    responseDTO.getFiPaidCost()));
            response = (ResponseDTO) responseDTO;
        }

        // 收益预估信息
        else if (param.getTableOrder() != null && param.getTableOrder().equals(10)) {
            ResponseDTO responseDTO = new ResponseDTO();
            SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM");
            SimpleDateFormat sdf2 = new SimpleDateFormat("yyyy-MM-00 00:00:00");
            Calendar cal = Calendar.getInstance();
            BaseInfoDTO baseInfo = (BaseInfoDTO) getBaseInfo(param).getData();

            ResponseDTO assetTotalInOutInfo;
            String oPTime = null;
            Date now = new Date();
            if (baseInfo != null) {
                // 计算每个月的净收入
                oPTime = baseInfo.getOperationTime();
                Date startTime = sdf1.parse(baseInfo.getOperationTime());
                Date endTime;
                Date lastTime = sdf1.parse(sdf1.format(now));

                if (param.getTimeStart() != null && !param.getTimeStart().equals("")) {
                    startTime = sdf1.parse(sdf1.format(param.getTimeStart()));
                }
                if (param.getTimeEnd() != null && !param.getTimeEnd().equals("")) {
                    lastTime = sdf1.parse(sdf1.format(param.getTimeEnd()));
                }
                if (StringUtils.hasLength(oPTime)) {
                    while (startTime.getTime() <= lastTime.getTime()) {
                        param.setTableOrder(9);
                        // 日期+1月
                        cal.setTime(startTime);
                        cal.add(Calendar.MONTH, 1);
                        endTime = cal.getTime();
                        param.setTimeStart(sdf2.parse(sdf2.format(startTime)));
                        param.setTimeEnd(sdf2.parse(sdf2.format(endTime)));
                        assetTotalInOutInfo = (ResponseDTO) getAssetProfitInfo(param).getData();
                        ResponseDTO.ProfitPredict profitPredict = new ResponseDTO.ProfitPredict();
                        profitPredict.setTime(sdf1.format(startTime));
                        profitPredict.setNetIncome(assetTotalInOutInfo.getNetIncome());
                        responseDTO.getProfitPredictInfoList().add(profitPredict);
                        startTime = cal.getTime();
                    }
                }
                // 计算已发生的月份数
                List<ResponseDTO.ProfitPredict> profitPredictInfoList = responseDTO.getProfitPredictInfoList();
                Calendar cal1 = Calendar.getInstance();
                Calendar cal2 = Calendar.getInstance();
                startTime = sdf1.parse(baseInfo.getOperationTime());
                endTime = sdf1.parse(sdf1.format(now));
                cal1.setTime(endTime);
                cal2.setTime(startTime);
                int surplus = cal2.get(Calendar.DATE) - cal1.get(Calendar.DATE);
                int result = cal2.get(Calendar.MONTH) - cal1.get(Calendar.MONTH);
                int month = cal2.get(Calendar.YEAR) - cal1.get(Calendar.YEAR);
                surplus = surplus <= 0 ? 1 : 0;
                int monthCount = Math.abs(month + result) + surplus;

                if (monthCount > 0 ) {

                    String irr;
                    List<BigDecimal> incomes = new ArrayList<>();
                    Integer count = 0;
                    // 计算当前IRR
                    for (ResponseDTO.ProfitPredict p : profitPredictInfoList) {
                        incomes.add(p.getNetIncome());
                        irr = IrrUtil.main(param.getTotalCost(), incomes);
                        responseDTO.getProfitPredictInfoList().get(count).setCurIRR(irr);
                        count = count + 1;
                    }
                    // 计算预估IRR
                    int ite = 0;
                    count = 0;
                    for (ResponseDTO.ProfitPredict p : profitPredictInfoList) {
                        if (count > 0) {
                            if (count <= monthCount) {  // 当前月份 已经发生
                                ite = count + 1;
                            } else {   // 当前月份 还未发生
                                ite = monthCount;
                            }
                            List<BigDecimal> preIncomes = new ArrayList<>();
                            for (int i = 0; i < ite; i++) {
                                preIncomes.add(incomes.get(i));
                            }
                            // 计算预估收入
                            for (int i = ite; i < 36; i++) {
                                if (i == ite && i == 1) {
                                    preIncomes.add(preIncomes.get(i - 1));
                                } else if (i == ite && i == 2) {
                                    preIncomes.add((preIncomes.get(i - 1).add(preIncomes.get(i - 2))).divide(new BigDecimal(2)));
                                } else {
                                    preIncomes.add((preIncomes.get(i - 1).add(preIncomes.get(i - 2)).add(preIncomes.get(i - 3))).divide(new BigDecimal(3)));
                                }
                            }
                            irr = IrrUtil.main(param.getTotalCost(), preIncomes);
                            responseDTO.getProfitPredictInfoList().get(count).setPreIRR(irr);

                            // 回本周期
                            BigDecimal accum = new BigDecimal(0);
                            for (int i = 1; i < count; i++) {
                                accum = accum.add(preIncomes.get(i));
                            }
                            responseDTO.getProfitPredictInfoList().get(count).setBackPeriod(Integer.parseInt(param.getTotalCost().subtract(accum).divide(preIncomes.get(count)).setScale(0, BigDecimal.ROUND_UP).toString()));
                            count = count + 1;
                        }
                    }
                }
            }
            response = (ResponseDTO) responseDTO;

        }
        return ApiDTO.ok("成功", response);
    }

    @ApiOperation(value = "其他信息绘图")
    @RequestMapping(value = "getProfitInfoFigure", method = RequestMethod.POST)
    public ApiDTO getAssetProfitInfoFigure(@RequestBody @ApiParam(name = "设备信息对象", value = "传入json格式", required = true) AssetProfitParam param) throws Exception {

        if (!StringUtils.hasLength(param.getSn())) {
            throw new ApiRuntimeException("请输入sn");
        }
        // 租赁信息
        Object response = null;
        if (param.getTableOrder() != null && param.getTableOrder().equals(1)) {
            param.setFigureOrNot(true);
            param.setIndex(0);
            param.setLength(0);
            Object rentInfoFigureDTO = appAssetService.getRentInfo(param);
            response = (FigureDTO) rentInfoFigureDTO;
        }

        // 营销信息
        if (param.getTableOrder() != null && param.getTableOrder().equals(4)) {
            param.setFigureOrNot(true);
            param.setIndex(0);
            param.setLength(0);
            Object rentInfoDTO = appAssetService.getSaleInfo(param);
            response = (FigureDTO) rentInfoDTO;
        }

        return ApiDTO.ok("成功", response);
    }

    @ApiOperation(value = "导出")
    @GetMapping("export")
    public void export(HttpServletResponse response, @RequestParam String sn, @RequestParam String timeStart, @RequestParam String timeEnd, @RequestParam Integer tableOrder, @RequestParam String type) throws ParseException {

        AssetProfitParam param = new AssetProfitParam();
        param.setSn(sn.equals("null") ? null : sn);
        param.setTimeStart(timeStart.equals("null") ? null : new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse(timeStart));
        param.setTimeEnd(timeEnd.equals("null") ? null : new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse(timeEnd));
        param.setTableOrder(tableOrder == null ? null : tableOrder);
        param.setType(type.equals("null") ? null : type);
        if (!StringUtils.hasLength(param.getSn())) {
            throw new ApiRuntimeException("请输入sn");
        }
        try {
            // 租赁信息

            if (param.getTableOrder() != null && param.getTableOrder().equals(1)) {
                param.setFigureOrNot(false);
                ResponseDTO responseDTO = (ResponseDTO) appAssetService.getRentInfo(param);
                List<Map<String, Object>> data = new ArrayList<>();
                responseDTO.getAllInfoList().forEach(p -> {
                    data.add(new HashMap<String, Object>() {{
                        put("time", p.getTime());
                        put("type", p.getType());
                        put("amount", p.getAmount());
                        put("orderNo", p.getOrder());
                        put("remark", p.getDescription());
                    }});
                });
                sysExcelFieldService.outExcelFile(sysExcelFieldService.export("rent_info", data), response, "租赁信息.xlsx");
            }
            if (param.getTableOrder() != null && (param.getTableOrder().equals(2) || param.getTableOrder().equals(3)
                    || param.getTableOrder().equals(5) || param.getTableOrder().equals(6)
                    || param.getTableOrder().equals(7) || param.getTableOrder().equals(8))) {
                AssetPageQueryParam appAssetConf = new AssetPageQueryParam();
                appAssetConf.setSn(param.getSn());
                appAssetConf.setTableOrder(param.getTableOrder());
                appAssetConf.setTimeStart(param.getTimeStart());
                appAssetConf.setTimeEnd(param.getTimeEnd());
                appAssetConf.setIndex(param.getIndex());
                appAssetConf.setLength(param.getLength());
                appAssetConf.setType(param.getType());
                Page<AppAssetConf> page = new Page<>(appAssetConf.getIndex(), appAssetConf.getLength());
                QueryWrapper<AppAssetConf> appAssetConfQueryWrapper = new QueryWrapper<>();
                appAssetConfQueryWrapper.eq("is_delete", 0);
                if (StringUtils.hasLength(appAssetConf.getSn())) {
                    appAssetConfQueryWrapper.like("sn", appAssetConf.getSn());
                }
                if (appAssetConf.getTableOrder() != null) {
                    appAssetConfQueryWrapper.eq("table_order", appAssetConf.getTableOrder());
                }
                if (appAssetConf.getType() != null) {
                    appAssetConfQueryWrapper.eq("type", appAssetConf.getType());
                }
                if (appAssetConf.getTimeStart() != null) {
                    appAssetConfQueryWrapper.gt("time", appAssetConf.getTimeStart());
                }
                if (appAssetConf.getTimeEnd() != null) {
                    appAssetConfQueryWrapper.lt("time", appAssetConf.getTimeEnd());
                }
                Page<AppAssetConf> appAssetConfPage = appAssetConfService.page(page, appAssetConfQueryWrapper);
                List<Map<String, Object>> data = new ArrayList<>();
                if (appAssetConfPage.getTotal() > 0) {
                    appAssetConfPage.getRecords().forEach(p -> {
                        data.add(new HashMap<String, Object>() {{
                            put("time", p.getTime());
                            put("type", p.getType());
                            put("amount", p.getAmount());
                            put("remark", "");
                        }});
                    });
                }
                switch (param.getTableOrder()) {
                    case 2:
                        sysExcelFieldService.outExcelFile(sysExcelFieldService.export("om_info", data), response, "运维信息.xlsx");
                        break;
                    case 3:
                        sysExcelFieldService.outExcelFile(sysExcelFieldService.export("com_info", data), response, "赔偿信息.xlsx");
                        break;
                    case 5:
                        sysExcelFieldService.outExcelFile(sysExcelFieldService.export("op_info", data), response, "运营信息.xlsx");
                        break;
                    case 6:
                        sysExcelFieldService.outExcelFile(sysExcelFieldService.export("fi_info", data), response, "金融成本信息.xlsx");
                        break;
                    case 7:
                        sysExcelFieldService.outExcelFile(sysExcelFieldService.export("handle_info", data), response, "处置信息.xlsx");
                        break;
                    case 8:
                        sysExcelFieldService.outExcelFile(sysExcelFieldService.export("other_info", data), response, "其他信息.xlsx");
                        break;

                    default:
                        break;
                }

            }
        } catch (Exception e) {
            throw new ApiRuntimeException("模板下载失败");

//        return ApiDTO.ok("成功");
        }
    }


}