package cn.fastfun.controller.api;


import cn.fastfun.controller.dto.Option;
import cn.fastfun.controller.param.DictQueryParam;
import cn.fastfun.controller.param.RoleTree;
import cn.fastfun.controller.param.SysRoleQueryParam;
import cn.fastfun.service.SysService;
import cn.fastfun.service.UtilService;
import cn.fastfun.service.entity.*;
import com.bridge.dto.*;
import com.bridge.exception.ApiRuntimeException;
import com.bridge.service.JpaService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.*;

@Api(tags = {"1.1 系统管理"})
@RestController
@Slf4j
@RequestMapping("/api/v1/sys")
public class SysController {

    //业务类
    @Resource(name = "sysDictService")
    JpaService<SysDict, String> sysDictService;


    //业务类
    @Resource(name = "sysRoleService")
    JpaService<SysRole, String> sysRoleService;

    //业务类
    @Resource(name = "sysRoleComponentService")
    JpaService<SysRoleComponent, String> sysRoleComponentService;

    @Resource
    SysService sysService;

    //用户
    @Resource(name = "sysUserService")
    JpaService<SysUser, String> sysUserService;

    //用户角色
    @Resource(name = "sysUserRoleService")
    JpaService<SysUserRole, String> sysUserRoleService;

    @Resource
    UtilService utilService;

    /**
     * 数据保存
     */
    @ApiOperation(value = "保存字典信息")
    @RequestMapping(value = "dict/save", method = RequestMethod.POST)
    public ApiDTO saveDict(@RequestBody @ApiParam(name = "字典表对象", value = "传入json格式", required = true) SysDict entity) {
        return ApiDTO.ok("保存成功", sysDictService.save(entity));
    }

    @ApiOperation(value = "查询字典信息")
    @RequestMapping(value = "dict/get", method = RequestMethod.POST)
    public ApiDTO getDict(@RequestBody @ApiParam(name = "字典表id", required = true) IdParam param) {
        return ApiDTO.ok("查询成功！", sysDictService.get(param.getId()));
    }

    @ApiOperation(value = "删除字典信息")
    @RequestMapping(value = "dict/delete", method = RequestMethod.POST)
    public ApiDTO deleteDict(@RequestBody @ApiParam(name = "字典表id", required = true) IdParam param) {
        sysDictService.delete(Arrays.asList(param.getId().split(",")));
        return ApiDTO.ok("删除成功！");
    }

    @ApiOperation(value = "分页搜索")
    @RequestMapping(value = "dict/pageQuery", method = RequestMethod.POST)
    public ApiPageDTO pageQueryDict(@RequestBody DictQueryParam param) {
        return new ApiPageDTO(null, sysDictService.findByParam(param));
    }

    @ApiOperation(value = "字典下拉选择")
    @RequestMapping(value = "dict/select", method = RequestMethod.POST)
    public ApiDTO selectDict(@RequestBody DictQueryParam param) {
        List<Option> options = new ArrayList<>();
        List<SysDict> list = sysDictService.findAll(param.getParam());
        if (!CollectionUtils.isEmpty(list))
            list.forEach(p -> options.add(new Option(p.getValue(), p.getName())));
        return ApiDTO.ok(options);
    }

    @ApiOperation(value = "所有字典")
    @RequestMapping(value = "dict/select/group", method = RequestMethod.POST)
    public ApiDTO selectDictGroup() {
        List<Option> options = new ArrayList<>();
        List<SysDict> list = sysDictService.findAll(QueryParamExp.desc("groupTitle"), QueryParamExp.group("groupName"));
        if (!CollectionUtils.isEmpty(list))
            list.forEach(p -> options.add(new Option(p.getGroupTitle(), p.getGroupName())));
        return ApiDTO.ok(options);
    }


    @ApiOperation(value = "角色分页搜索")
    @RequestMapping(value = "role/pageQuery", method = RequestMethod.POST)
    public ApiPageDTO pageQueryRole(@RequestBody SysRoleQueryParam param) {
        return new ApiPageDTO(null, sysRoleService.findByParam(param));
    }

    /**
     * 数据保存
     */
    @ApiOperation(value = "保存角色信息")
    @RequestMapping(value = "role/save", method = RequestMethod.POST)
    public ApiDTO saveRole(@RequestBody @ApiParam(name = "权限角色对象", value = "传入json格式", required = true) SysRole entity) {
        return ApiDTO.ok(sysRoleService.save(entity));
    }

    @ApiOperation(value = "查询角色信息")
    @RequestMapping(value = "role/get", method = RequestMethod.POST)
    public ApiDTO getRole(@RequestBody @ApiParam(name = "角色ID", required = true) IdParam id) {
        return ApiDTO.ok(sysRoleService.get(id.getId()));
    }

    @ApiOperation(value = "删除角色信息")
    @RequestMapping(value = "role/delete", method = RequestMethod.POST)
    public ApiDTO deleteRole(@RequestBody @ApiParam(name = "角色ID", required = true) IdParam id) {
        SysRole sysRole = sysRoleService.get(id.getId());
        if (null != sysRole) {
            List<SysUserRole> userRoles = sysUserRoleService.findAll(QueryParamExp.eq("roleCode", sysRole.getCode()));
            if (userRoles.size() > 0) return ApiDTO.error("角色下存在账号，请先删除账号!");
            sysRoleService.delete(id.getId());
        }
        return ApiDTO.ok("删除成功！");
    }


    @ApiOperation(value = "角色下拉集合")
    @RequestMapping(value = "role/select", method = RequestMethod.POST)
    public ApiDtDTO allRole() {
        List<Option> options = new ArrayList<>();
        List<SysRole> list = sysRoleService.findAll();
        if (!CollectionUtils.isEmpty(list)) list.forEach(p -> options.add(new Option(p.getTitle(), p.getCode())));
        return new ApiDtDTO("", options);
    }

    @ApiOperation(value = "登陆用户权限")
    @RequestMapping(value = "logInrole/tree", method = RequestMethod.POST)
    public ApiDTO logInRoleTree(@RequestBody IdParam param) {
        if (StringUtils.isEmpty(param.getId())) throw new ApiRuntimeException("没有角色编码");
        SysUserRole sysUserRole = sysUserRoleService.getOne(QueryParamExp.eq("userId", param.getId()));
        SysRole sysRole = sysRoleService.getOne(QueryParamExp.eq("code", sysUserRole.getRoleCode()));
        if (null == sysRole) throw new ApiRuntimeException("没有找对应角色");
        Map<String, Object> objectMap = new HashMap<>();
        objectMap.put("name", sysRole.getTitle());

        List<SysRoleComponent> sysRoleComponentList = sysRoleComponentService.findAll(QueryParamExp.eq("roleCode", sysUserRole.getRoleCode()));
        List<String> checked = new ArrayList<>();
        sysRoleComponentList.forEach(p -> {
                checked.add(p.getComponentId());
        });
        objectMap.put("checked", checked);
        return ApiDTO.ok(null, objectMap);
    }

    @ApiOperation(value = "权限菜单树")
    @RequestMapping(value = "role/tree", method = RequestMethod.POST)
    public ApiDTO menuTree(@RequestBody IdParam param) {

        Map<String, Object> objectMap = new HashMap<>();

        List<SysRoleComponent> sysRoleComponentList = sysRoleComponentService.findAll(QueryParamExp.eq("roleCode", param.getId()));
        List<String> checked = new ArrayList<>();
        sysRoleComponentList.forEach(p -> {
            checked.add(p.getComponentId());
        });
        objectMap.put("checked", checked);
        return ApiDTO.ok(null, objectMap);
    }

    /**
     * 数据保存
     */
    @ApiOperation(value = "保存信息")
    @RequestMapping(value = "role/user/save", method = RequestMethod.POST)
    public ApiDTO save(@RequestBody @ApiParam(name = "角色组件对象", value = "传入json格式", required = true) RoleTree param) {
        sysService.saveRoleMenu(param);
        return ApiDTO.ok("保存成功");
    }

}
