package cn.fastfun.service.impl;

import cn.fastfun.controller.dto.asset.*;
import cn.fastfun.controller.param.*;
import cn.fastfun.util.OkHttpCli;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.Future;

@Slf4j
@Service
public class AppAssetProfitServiceImpl {
    @Autowired
    OkHttpCli okHttpCli;

    @Value("${openapi.ip}")
    private String openApiIp;

    @Value("${openapi.port}")
    private String openApiPort;


    // 请求开放平台
    public List<String> reqOpenApi(List<Future<String>> futures) throws Exception {
        List<String> response = new ArrayList<>();
        try {
            Boolean doneFlag = false;
            //死循环，每隔2000ms执行一次，判断一下这三个异步调用的方法是否全都执行完了。
            while (true) {
                //使用Future的isDone()方法返回该方法是否执行完成
                for (Future f : futures) {
                    if (!f.isDone())
                        break;
                    doneFlag = true;
                }
                if (doneFlag)
                    break;
                Thread.sleep(200);
            }
            for (Future<String> future : futures) {
                String string = future.get();
                response.add(string);
            }
        } catch (Exception e) {
            throw new Exception("" + e.getMessage());
        }
        return response;
    }

    // 拼接异步请求参数
    @Async
    public Future<String> asyncReq(JSONObject param, String url, Map<String, String> mapHeaders ) {
        mapHeaders.put("token", "2A26DD0ADE53456D928562A17131A3B5");
        String response = okHttpCli.doPostTokenJson(String.format(url, openApiIp, openApiPort), param.toJSONString(), mapHeaders);
        Future<String> result = new AsyncResult<>(response);
        return result;
    }

    // 资产基础信息
    public BaseInfoDTO getBaseInfo(AssetProfitParam param) throws Exception {
        BaseInfoDTO baseInfo = new BaseInfoDTO();

        // 请求开放平台
        List<Future<String>> futures = new ArrayList<>();

        JSONObject json = new JSONObject();
        json.put("sn", param.getSn());
        futures.add(asyncReq(json, "http://%s:%s/admin/v1/assetBatteryBasicInfo", new HashMap<>()));
        futures.add(asyncReq(json, "http://%s:%s/admin/v1/getDateLine", new HashMap<>()));
        List<String> response = reqOpenApi(futures);

        // 开放平台返回数据解析
        JSONObject baseData = JSONObject.parseObject(response.get(0));
        BasicAssetDto basicAssetDto = new BasicAssetDto();
        if (baseData != null && baseData.get("data") != null) {
            basicAssetDto = JSONObject.parseObject(baseData.get("data").toString(), BasicAssetDto.class);
        }

        baseInfo.setSn(basicAssetDto.getSn());
        baseInfo.setCategory(basicAssetDto.getDeviceTypeText());
        baseInfo.setType(basicAssetDto.getProductTypeText());
        baseInfo.setFactory(basicAssetDto.getFactory());
        baseInfo.setCellFactory(basicAssetDto.getElectricCore());
        baseInfo.setCellType(basicAssetDto.getElectricCoreTypeText());
        baseInfo.setVoltPlatform(basicAssetDto.getVoltageText());
        baseInfo.setCapacity(basicAssetDto.getCapacity());
        baseInfo.setProductTime(basicAssetDto.getCreateTime());
        baseInfo.setOperationTime(basicAssetDto.getOperationTime());
        baseInfo.setPrice(basicAssetDto.getPurchasePrice());
        baseInfo.setStatus(basicAssetDto.getStatusText());
        baseInfo.setRegion(basicAssetDto.getLocationText());
        JSONObject dataLine = JSONObject.parseObject(response.get(1));
        List<DeviceLineDto> deviceLineDtos = new ArrayList<>();
        if (dataLine != null && dataLine.get("data") != null) {
            deviceLineDtos = JSONArray.parseArray(dataLine.get("data").toString(), DeviceLineDto.class);
        }

        // 返回时间线
        BaseInfoDTO.TimeLine deliverTimeline = new BaseInfoDTO.TimeLine();
        deliverTimeline.setTime("发货时间");
        BaseInfoDTO.TimeLine storageTimeline = new BaseInfoDTO.TimeLine();
        storageTimeline.setTime("入库时间");
        BaseInfoDTO.TimeLine rentTimeline = new BaseInfoDTO.TimeLine();
        rentTimeline.setTime("首次租赁时间");
        BaseInfoDTO.TimeLine recallTimeline = new BaseInfoDTO.TimeLine();
        recallTimeline.setTime("回调时间");

        Boolean storageFlag = false;
        Boolean rentFlag = false;
        if (deviceLineDtos.size() > 0) {
            Collections.reverse(deviceLineDtos);
            for (DeviceLineDto d : deviceLineDtos) {
                // 入库时间
                if (d.getAction().equals("20") && !storageFlag) {
                    deliverTimeline.setTime(d.getActionTime());
                    storageFlag = true;
                }
                // 首次租赁时间
                if ((d.getAction().equals("7") || d.getAction().equals("8")) && !rentFlag) {
                    rentTimeline.setTime(d.getActionTime());
                    rentFlag = true;
                }
                // 回调时间
                if (d.getAction().equals("22")) {
                    recallTimeline.setTime(d.getActionTime());

                }
            }
        }
        baseInfo.getTimeLine().add(new BaseInfoDTO.TimeLine() {{
            setTime(baseInfo.getProductTime());
            setTime("生产时间");
        }});
        baseInfo.getTimeLine().add(deliverTimeline);
        baseInfo.getTimeLine().add(storageTimeline);
        baseInfo.getTimeLine().add(new BaseInfoDTO.TimeLine() {{
            setTime(baseInfo.getOperationTime());
            setTime("投营时间");
        }});
        baseInfo.getTimeLine().add(rentTimeline);
        baseInfo.getTimeLine().add(recallTimeline);
        // 采购和投营时间
        if (baseInfo.getOperationTime() != null) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Calendar cal = Calendar.getInstance();
            cal.setTime(sdf.parse(baseInfo.getOperationTime()));
            long time1 = cal.getTimeInMillis();
            cal.setTime(new Date());
            long time2 = cal.getTimeInMillis();
            long between_days = (time2 - time1) / (1000 * 3600 * 24);
            baseInfo.setOperationDay(between_days);
        }
//        baseInfo.setDeliverTime();
        return baseInfo;
    }

    // 租赁信息
    public Object getRentInfo(AssetProfitParam param) throws Exception {
        Object result = null;

        // 请求开放平台
        List<Future<String>> futures = new ArrayList<>();
        JSONObject json = new JSONObject();
        List<String> response = new ArrayList<>();

        // 请求的是列表
        if (param.getFigureOrNot() == false) {
            // 请求开放平台
            json.put("sn", param.getSn());
            json.put("pageSize", param.getLength());
            json.put("pageIndex", param.getIndex());
            if (param.getType() == null || param.getType() == "") {
                json.put("type", "1,2,3,4,5");
            } else {
                json.put("type", param.getType());
            }
            futures.add(asyncReq(json, "http://%s:%s/admin/v1/getDepositAndEnt", new HashMap<>()));
            response = reqOpenApi(futures);

            // 解析开放平台返回数据
            ResponseDTO responseDTO = new ResponseDTO();
            JSONObject rentInfo = JSONObject.parseObject(response.get(0));
            SummaryCentAndDepositDto summaryCentAndDepositDto = new SummaryCentAndDepositDto();
            List<RentAndDepositDto> rentAndDepositDtos = new ArrayList<>();
            if (rentInfo.get("data") != null && rentInfo.get("data") != "") {
                JSONObject responseData = JSONObject.parseObject(rentInfo.get("data").toString());
                if (responseData.get("summary") != null && responseData.get("summary") != "") {
                    summaryCentAndDepositDto = JSONObject.parseObject(responseData.get("summary").toString(), SummaryCentAndDepositDto.class);
                }
                if (responseData.get("data") != null && responseData.get("data") != "") {
                    rentAndDepositDtos = JSONArray.parseArray(responseData.get("data").toString(), RentAndDepositDto.class);
                }
            }
            for (RentAndDepositDto r : rentAndDepositDtos) {
                ResponseDTO.AllInfo allInfo = new ResponseDTO.AllInfo();
                allInfo.setTime(r.getOrderTime());
                allInfo.setOrder(r.getOrderNo());
                allInfo.setDescription(r.getRemark());
                allInfo.setAmount(r.getAmount());
                switch (r.getType()) {
                    case 1:
                        allInfo.setType("押金-收");
                        break;
                    case 2:
                        allInfo.setType("押金-支");
                        break;
                    case 3:
                        allInfo.setType("租金-收");
                        break;
                    case 4:
                        allInfo.setType("租金-支");
                        break;
                    case 5:
                        allInfo.setType("丢失保障金");
                        break;
                    default:
                        allInfo.setType("");
                }
                responseDTO.getAllInfoList().add(allInfo);
            }
            responseDTO.setOrderStartTime(summaryCentAndDepositDto.getOrderStartTime());
            responseDTO.setOrderEndTime(summaryCentAndDepositDto.getOrderEndTime());
            responseDTO.setRental(summaryCentAndDepositDto.getCurrentCent());
            responseDTO.setCash(summaryCentAndDepositDto.getCurrentDeposit());
            responseDTO.setIncome(summaryCentAndDepositDto.getCent());
            responseDTO.setCashIn(summaryCentAndDepositDto.getDeposit());
            responseDTO.setSecurityCash(summaryCentAndDepositDto.getSecurity());
            responseDTO.setNetRental(summaryCentAndDepositDto.getNetIncome());
            result = (ResponseDTO) responseDTO;
        }
        // 请求的是图表
        else {
            // 请求开放平台
            json.put("sn", param.getSn());
            json.put("type", "1,2,3,4,5");
            json.put("startTime", param.getTimeStart() == null ? null : new SimpleDateFormat("yyyy-MM-dd").format(param.getTimeStart()));
            json.put("endTime", param.getTimeEnd() == null ? null : new SimpleDateFormat("yyyy-MM-dd").format(param.getTimeEnd()));
            futures.add(asyncReq(json, "http://%s:%s/admin/v1/getDepositAndEntChart", new HashMap<>()));
            response = reqOpenApi(futures);

            FigureDTO figureDTO = new FigureDTO();
            JSONObject responseData = JSONObject.parseObject(response.get(0));
            if (responseData.get("data") != null && responseData.get("data") != "") {
                JSONObject rentFigureInfo = JSONObject.parseObject(responseData.get("data").toString());
                Boolean timeListDone = false;
                List<String> timeList = new ArrayList<>();
                if (rentFigureInfo.get("securityChartList") != null && rentFigureInfo.get("securityChartList") != "") {
                    List<RentAndDepositChartDto> chartList = new ArrayList<>();
                    chartList = JSONArray.parseArray(rentFigureInfo.get("securityChartList").toString(), RentAndDepositChartDto.class);
                    if (chartList.size() > 0) {
                        figureDTO.getLegendList().add("丢失保障金");
                        List<BigDecimal> dataList = new ArrayList<>();
                        if (!timeListDone) {
                            chartList.forEach(p -> timeList.add(p.getMonth()));
                            timeListDone = true;
                        }
                        chartList.forEach(p -> dataList.add(p.getAmountMonth()));
                        figureDTO.getDataList().add(dataList);
                    }
                }
                if (rentFigureInfo.get("centReduceChartList") != null && rentFigureInfo.get("centReduceChartList") != "") {
                    List<RentAndDepositChartDto> chartList = new ArrayList<>();
                    chartList = JSONArray.parseArray(rentFigureInfo.get("centReduceChartList").toString(), RentAndDepositChartDto.class);
                    if (chartList.size() > 0) {
                        figureDTO.getLegendList().add("租金-支");
                        List<BigDecimal> dataList = new ArrayList<>();
                        if (!timeListDone) {
                            chartList.forEach(p -> timeList.add(p.getMonth()));
                            timeListDone = true;
                        }
                        chartList.forEach(p -> dataList.add(p.getAmountMonth()));
                        figureDTO.getDataList().add(dataList);
                    }
                }
                if (rentFigureInfo.get("centAddChartList") != null && rentFigureInfo.get("centAddChartList") != "") {
                    List<RentAndDepositChartDto> chartList = new ArrayList<>();
                    chartList = JSONArray.parseArray(rentFigureInfo.get("centAddChartList").toString(), RentAndDepositChartDto.class);
                    if (chartList.size() > 0) {
                        figureDTO.getLegendList().add("租金-收");
                        List<BigDecimal> dataList = new ArrayList<>();
                        if (!timeListDone) {
                            chartList.forEach(p -> timeList.add(p.getMonth()));
                            timeListDone = true;
                        }
                        chartList.forEach(p -> dataList.add(p.getAmountMonth()));
                        figureDTO.getDataList().add(dataList);
                    }
                }
                if (rentFigureInfo.get("depositReduceChartList") != null && rentFigureInfo.get("depositReduceChartList") != "") {
                    List<RentAndDepositChartDto> chartList = new ArrayList<>();
                    chartList = JSONArray.parseArray(rentFigureInfo.get("depositReduceChartList").toString(), RentAndDepositChartDto.class);
                    if (chartList.size() > 0) {
                        figureDTO.getLegendList().add("押金-支");
                        List<BigDecimal> dataList = new ArrayList<>();
                        if (!timeListDone) {
                            chartList.forEach(p -> timeList.add(p.getMonth()));
                            timeListDone = true;
                        }
                        chartList.forEach(p -> dataList.add(p.getAmountMonth()));
                        figureDTO.getDataList().add(dataList);
                    }
                }
                if (rentFigureInfo.get("depositAddChartList") != null && rentFigureInfo.get("depositAddChartList") != "") {
                    List<RentAndDepositChartDto> chartList = new ArrayList<>();
                    chartList = JSONArray.parseArray(rentFigureInfo.get("depositAddChartList").toString(), RentAndDepositChartDto.class);
                    if (chartList.size() > 0) {
                        figureDTO.getLegendList().add("押金-收");
                        List<BigDecimal> dataList = new ArrayList<>();
                        if (!timeListDone) {
                            chartList.forEach(p -> timeList.add(p.getMonth()));
                            timeListDone = true;
                        }
                        chartList.forEach(p -> dataList.add(p.getAmountMonth()));
                        figureDTO.getDataList().add(dataList);
                    }
                }
                figureDTO.setTimeList(timeList);
            }
            result = (FigureDTO) figureDTO;

        }
        return result;
    }

    // 营销信息
    public Object getSaleInfo(AssetProfitParam param) throws Exception {
        Object result = null;

        // 请求开放平台
        List<Future<String>> futures = new ArrayList<>();
        JSONObject json = new JSONObject();
        List<String> response = new ArrayList<>();

        // 请求的是列表
        if (param.getFigureOrNot() == false) {
            // 请求开放平台
            json.put("sn", param.getSn());
            json.put("pageSize", param.getLength());
            json.put("pageIndex", param.getIndex());
            if (param.getType() == null || param.getType() == "") {
                json.put("type", "1");
            } else {
                json.put("type", param.getType());
            }
            futures.add(asyncReq(json, "http://%s:%s/admin/v1/marketingInformation", new HashMap<>()));
            response = reqOpenApi(futures);

            // 解析开放平台返回数据
            ResponseDTO responseDTO = new ResponseDTO();
            JSONObject rentInfo = JSONObject.parseObject(response.get(0));
            SummaryCentAndDepositDto summaryCentAndDepositDto = new SummaryCentAndDepositDto();
            List<MarketingInformationDto> rentAndDepositDtos = new ArrayList<>();
            if (rentInfo.get("data") != null && rentInfo.get("data") != "") {
                JSONObject responseData = JSONObject.parseObject(rentInfo.get("data").toString());
                if (responseData.get("summary") != null && responseData.get("summary") != "") {
                    summaryCentAndDepositDto = JSONObject.parseObject(responseData.get("summary").toString(), SummaryCentAndDepositDto.class);
                }
                if (responseData.get("data") != null && responseData.get("data") != "") {
                    rentAndDepositDtos = JSONArray.parseArray(responseData.get("data").toString(), MarketingInformationDto.class);
                }
            }
            for (MarketingInformationDto r : rentAndDepositDtos) {
                ResponseDTO.AllInfo allInfo = new ResponseDTO.AllInfo();
                allInfo.setTime(new SimpleDateFormat("yyyy-MM-dd").format(r.getUsedTime()));
                allInfo.setDescription(r.getRemark());
                allInfo.setAmount(r.getAmount());
                switch (r.getType()) {
                    case 1:
                        allInfo.setType("活动金额");
                        break;
                    default:
                        allInfo.setType("");
                }
                responseDTO.getAllInfoList().add(allInfo);
            }
            responseDTO.setOrderStartTime(summaryCentAndDepositDto.getOrderStartTime());
            responseDTO.setOrderEndTime(summaryCentAndDepositDto.getOrderEndTime());
            responseDTO.setRental(summaryCentAndDepositDto.getCurrentCent());
            responseDTO.setCash(summaryCentAndDepositDto.getCurrentDeposit());
            responseDTO.setIncome(summaryCentAndDepositDto.getCent());
            responseDTO.setCashIn(summaryCentAndDepositDto.getDeposit());
            responseDTO.setSecurityCash(summaryCentAndDepositDto.getSecurity());
            responseDTO.setNetRental(summaryCentAndDepositDto.getNetIncome());
            result = (ResponseDTO) responseDTO;
        }
        // 请求的是图表
        else {
            // 请求开放平台
            json.put("sn", param.getSn());
            json.put("type", "1");
            json.put("startTime", param.getTimeStart() == null ? null : new SimpleDateFormat("yyyy-MM-dd").format(param.getTimeStart()));
            json.put("endTime", param.getTimeEnd() == null ? null : new SimpleDateFormat("yyyy-MM-dd").format(param.getTimeEnd()));
            futures.add(asyncReq(json, "http://%s:%s/admin/v1/marketChart", new HashMap<>()));
            response = reqOpenApi(futures);

            FigureDTO figureDTO = new FigureDTO();
            JSONObject responseData = JSONObject.parseObject(response.get(0));
            if (responseData.get("data") != null && responseData.get("data") != "") {
                JSONObject rentFigureInfo = JSONObject.parseObject(responseData.get("data").toString());
                Boolean timeListDone = false;
                List<String> timeList = new ArrayList<>();
                if (rentFigureInfo.get("couponList") != null && rentFigureInfo.get("couponList") != "") {
                    List<MarketChartDto> chartList = new ArrayList<>();
                    chartList = JSONArray.parseArray(rentFigureInfo.get("couponList").toString(), MarketChartDto.class);
                    if (chartList.size() > 0) {
                        figureDTO.getLegendList().add("活动金额");
                        List<BigDecimal> dataList = new ArrayList<>();
                        if (!timeListDone) {
                            chartList.forEach(p -> timeList.add(p.getMonth()));
                            timeListDone = true;
                        }
                        chartList.forEach(p -> dataList.add(p.getAmountMonth()));
                        figureDTO.getDataList().add(dataList);
                    }
                }
                figureDTO.setTimeList(timeList);
            }
            result = (FigureDTO) figureDTO;

        }
        return result;
    }

}
